@@ -152,22 +152,38 @@ static inline void sigdie_for_sig(int sig, int inside)
     /* Quit Python with an appropriate message.
        Make sure to check the standard signals from the C standard first,
        in case systems alias some of these constants. */
-    if (inside)
-        sigdie(sig, "An error occurred during signal handling.");
-    else if (sig == SIGILL)
-        sigdie(sig, "Unhandled SIGILL: An illegal instruction occurred.");
-    else if (sig == SIGABRT)
-        sigdie(sig, "Unhandled SIGABRT: An abort() occurred.");
-    else if (sig == SIGFPE)
-        sigdie(sig, "Unhandled SIGFPE: An unhandled floating point exception occurred.");
-    else if (sig == SIGSEGV)
-        sigdie(sig, "Unhandled SIGSEGV: A segmentation fault occurred.");
-    else if (sig == SIGBUS)
-        sigdie(sig, "Unhandled SIGBUS: A bus error occurred.");
-    else if (sig == SIGQUIT)
-        sigdie(sig, NULL);
-    else
-        sigdie(sig, "Unknown signal received.");
+    if (inside) {
+        if (sig == SIGILL)
+            sigdie(sig, "Unhandled SIGILL during signal handling.");
+        else if (sig == SIGABRT)
+            sigdie(sig, "Unhandled SIGABRT during signal handling.");
+        else if (sig == SIGFPE)
+            sigdie(sig, "Unhandled SIGFPE during signal handling.");
+        else if (sig == SIGSEGV)
+            sigdie(sig, "Unhandled SIGSEGV during signal handling.");
+        else if (sig == SIGBUS)
+            sigdie(sig, "Unhandled SIGBUS during signal handling.");
+        else if (sig == SIGQUIT)
+            sigdie(sig, NULL);
+        else
+            sigdie(sig, "Unknown signal during signal handling.");
+    }
+    else {
+        if (sig == SIGILL)
+            sigdie(sig, "Unhandled SIGILL: An illegal instruction occurred.");
+        else if (sig == SIGABRT)
+            sigdie(sig, "Unhandled SIGABRT: An abort() occurred.");
+        else if (sig == SIGFPE)
+            sigdie(sig, "Unhandled SIGFPE: An unhandled floating point exception occurred.");
+        else if (sig == SIGSEGV)
+            sigdie(sig, "Unhandled SIGSEGV: A segmentation fault occurred.");
+        else if (sig == SIGBUS)
+            sigdie(sig, "Unhandled SIGBUS: A bus error occurred.");
+        else if (sig == SIGQUIT)
+            sigdie(sig, NULL);
+        else
+            sigdie(sig, "Unknown signal received.");
+    }
 }
 
 
@@ -426,14 +442,21 @@ static void _sig_off_warning(const char* file, int line)
 static void setup_alt_stack(void)
 {
 #if HAVE_SIGALTSTACK
-    /* Static space for the alternate signal stack. The size should be
+    /* Space for the alternate signal stack. The size should be
      * of the form MINSIGSTKSZ + constant. The constant is chosen rather
-     * ad hoc but sufficiently large. */
-    static char alt_stack[MINSIGSTKSZ + 5120 + BACKTRACELEN * sizeof(void*)];
-
+     * ad hoc but sufficiently large.
+     * MINSIGSTKSZ is no longer a constant starting with glibc 2.34. */
     stack_t ss;
+#if MINSIGSTKSZ_IS_CONSTANT
+    static char alt_stack[MINSIGSTKSZ + 5120 + BACKTRACELEN * sizeof(void*)];
     ss.ss_sp = alt_stack;
     ss.ss_size = sizeof(alt_stack);
+#else
+    size_t stack_size = MINSIGSTKSZ + 5120 + BACKTRACELEN * sizeof(void*);
+    ss.ss_sp = malloc(stack_size);
+    ss.ss_size = stack_size;
+#endif
+    if (ss.ss_sp == NULL) {perror("cysignals malloc alt signal stack"); exit(1);}
     ss.ss_flags = 0;
     if (sigaltstack(&ss, NULL) == -1) {perror("cysignals sigaltstack"); exit(1);}
 #endif
